package com.mx.finger.dome;

import android.Manifest;
import android.app.ActionBar;
import android.content.pm.PackageManager;
import android.graphics.Bitmap;
import android.hardware.camera2.TotalCaptureResult;
import android.os.Build;
import android.os.Bundle;

import android.os.Handler;
import android.os.Message;
import android.text.Editable;
import android.util.Log;
import android.view.View;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.TextView;
import android.widget.Toast;

import androidx.annotation.Nullable;
import androidx.appcompat.app.AppCompatActivity;

import com.mx.finger.alg.MxIDFingerAlg;
import com.mx.finger.api.msc.MxIdMscFingerApiFactory;
import com.mx.finger.api.msc.MxMscBigFingerApi;
import com.mx.finger.common.MxImage;
import com.mx.finger.common.Result;
import com.mx.finger.utils.LogUtils;
import com.mx.finger.utils.RawBitmapUtils;
import com.rscja.deviceapi.UsbFingerprint;
import com.rscja.utility.StringUtility;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

public class MainActivity extends AppCompatActivity {

    private static final String TAG = "Mx-MainActivity";
    private static final int TIME_OUT = 5000;
    private final ExecutorService executor = Executors.newSingleThreadExecutor();

    private byte[] featureBufferEnroll = new byte[1];
    private byte[] featureBufferMatch = new byte[1];

    private MxMscBigFingerApi mxMscBigFingerApi;
    private MxIDFingerAlg mxFingerAlg;
    private SearchThread mSearchThread;

    private byte[] featTemp = null;
    private String featTempPath="sdcard/featData.txt";

    @Override
    protected void onCreate(@Nullable Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
         setContentView(R.layout.activity_main2);
         checkReadWritePermission();
         UsbFingerprint.getInstance().FingerprintSwitchUsb();
         UsbFingerprint.getInstance().UsbToFingerprint();
        // show logs
        ActionBar actionBar = getActionBar();
        if (actionBar != null) {
            actionBar.setTitle(R.string.app_name);
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (checkSelfPermission(Manifest.permission.READ_PHONE_STATE) != PackageManager.PERMISSION_GRANTED
                    || checkSelfPermission(Manifest.permission.READ_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
                requestPermissions(new String[]{Manifest.permission.READ_EXTERNAL_STORAGE, Manifest.permission.READ_PHONE_STATE}, 0x99);
            }
        }

        LogUtils.logAble(true);
        MxIdMscFingerApiFactory fingerFactory = new MxIdMscFingerApiFactory(getApplication());
        mxMscBigFingerApi = fingerFactory.getApi();
        mxFingerAlg = fingerFactory.getAlg();


        findViewById(R.id.getVersion).setOnClickListener(v -> executor.execute(() -> {
            String version = mxFingerAlg.version();
            showLog("GetVersion mxFingerAlg success \n" + version);
            Result<String> deviceInfo = mxMscBigFingerApi.getDeviceInfo();
            if (deviceInfo.isSuccess()) {
                showLog("GetVersion success \n" + deviceInfo.data);
            } else {
                showLog("GetVersion fail \nCode  : " + deviceInfo.code);
            }
        }));

        findViewById(R.id.getSerialNumber).setOnClickListener(v -> executor.execute(() -> {
            Result<String> result = mxMscBigFingerApi.getDeviceSerialNumber();
            if (result.isSuccess()) {
                showLog("GetSerialNumber success \n" + result.data);
            } else {
                showLog("GetSerialNumber fail \nCode  : " + result.code);
            }
        }));

        findViewById(R.id.detectFinger).setOnClickListener(v -> executor.execute(() -> {
            Result<Boolean> result = mxMscBigFingerApi.detectFinger();
            if (result.isSuccess()) {
                showLog("DetectFinger success \n" + (result.data != null && result.data ? " Has Finger" : "No Finger"));
            } else {
                showLog("DetectFinger fail \nCode  : " + result.code);
            }
        }));

        findViewById(R.id.getFingerImage).setOnClickListener(v -> executor.execute(() -> {
            showProgressDialog("Please press finger ");
            Result<MxImage> result = mxMscBigFingerApi.getFingerImageBig(TIME_OUT);
            if (result.isSuccess()) {
                showLog("GetFingerImage success !");
                showFingerImage(result.data);
            } else {
                showLog("GetFingerImage fail \nCode  : " + result.code);
                showFingerImage(null);
            }
            dismissProgressDialog();
        }));


        findViewById(R.id.enrollFingerHost).setOnClickListener(v -> executor.execute(() -> {
            showProgressDialog("Please press finger ");
            try {
                //step 0 get finger image
                Result<MxImage> result = mxMscBigFingerApi.getFingerImageBig(TIME_OUT);
                if (!result.isSuccess()) {
                    showLog("GetFingerImage fail \nCode  : " + result.code);
                    showFingerImage(null);
                    return;
                }
                //step 1 extract finger feature
                MxImage image = result.data;
                showFingerImage(image);
                assert image != null;
                featureBufferEnroll = mxFingerAlg.extractFeature(image.data, image.width, image.height);
                if (featureBufferEnroll == null) {
                    appendLog("EnrollFingerHost fail ");
                } else {
                    appendLog("EnrollFingerHost success ");
                }
            } finally {
                dismissProgressDialog();
            }
        }));

        findViewById(R.id.matchFingerHost).setOnClickListener(v -> executor.execute(() -> {
            showProgressDialog("Please press finger ");
            try {
                //step 0 get finger image
                Result<MxImage> result = mxMscBigFingerApi.getFingerImageBig(TIME_OUT);
                if (!result.isSuccess()) {
                    showLog("GetFingerImage fail \nCode : " + result.code);
                    showFingerImage(null);
                    return;
                }
                MxImage image = result.data;
                showFingerImage(image);
                //step 1 get finger feature
                assert image != null;
                featureBufferMatch = mxFingerAlg.extractFeature(image.data, image.width, image.height);
                if (featureBufferMatch == null) {
                    appendLog("ExtractFeature fail");
                    return;
                }
                //step 2 match finger feature
                int match = mxFingerAlg.match(featureBufferEnroll, featureBufferMatch, 3);
                if (match == 0) {
                    appendLog("MatchFingerFeature success");
                } else {
                    appendLog("MatchFingerFeature fail \nCode : " + match);
                }
            } finally {
                dismissProgressDialog();
            }
        }));


        findViewById(R.id.flashSize).setOnClickListener(v -> executor.execute(() -> {
            Result<int[]> result = mxMscBigFingerApi.getFlashSize();
            if (result.isSuccess()) {
                assert result.data != null;
                showLog(String.format("GetFlashSize success  \nRemaining size: %s \nAll size: %s  ", result.data[1], result.data[0]));
            } else {
                showLog("GetFlashSize fail \nCode : " + result.code);
            }
        }));

        findViewById(R.id.enrollTemplate).setOnClickListener(v -> executor.execute(() -> {
            showFingerImage(null);
            showProgressDialog("Please press finger");
            try {
                int position = getTemplatePosition();
                if (position < 0) {
                    return;
                }
                // if you wan't test download finger feature, use this line .
                // Result<Boolean> saveResult = enrollTemplateWithHostFeature(position);
                Result<Boolean> saveResult = mxMscBigFingerApi.extractFingerFeatureBig(position, MxMscBigFingerApi.FLASH, TIME_OUT);
                if (!saveResult.isSuccess()) {
                    showLog("EnrollTemplate failed \nCode : " + saveResult.code);
                } else {
                    showLog("EnrollTemplate success\nPosition : " + position);
                }
            } finally {
                dismissProgressDialog();
            }
        }));

        findViewById(R.id.updateTemplate).setOnClickListener(v -> executor.execute(() -> {
            showFingerImage(null);
            showProgressDialog("Please press finger");
            try {
                int position = getTemplatePosition();
                if (position < 0) {
                    return;
                }
                Result<byte[]> saveResult = mxMscBigFingerApi.getFingerFeatureBig(position, 1/*flash*/, 2000);
                if (!saveResult.isSuccess()) {
                    showLog("getFingerFeatureBig failed \nCode : " + saveResult.code);
                } else {
                   // featTemp = saveResult.data;
                    writeFeatTemp(saveResult.data);
                    showLog("getFingerFeatureBig success\nPosition : " + position);
                }
            } finally {
                dismissProgressDialog();
            }
        }));

        findViewById(R.id.downloadTemplate).setOnClickListener(v -> executor.execute(() -> {
            showFingerImage(null);
            showProgressDialog("Please press finger");
            try {
                int position = getTemplatePosition();
                if (position < 0) {
                    return;
                }
               byte[] data= readFeatTemp();
                if(data==null)
                    return;

                Result<Boolean> saveResult = mxMscBigFingerApi.downloadFlashTemplate(position, data);
                // if you wan't test download finger feature, use this line .
                // Result<Boolean> saveResult = enrollTemplateWithHostFeature(position);
                //Result<Boolean> saveResult = mxMscBigFingerApi.extractFingerFeatureBig(position, MxMscBigFingerApi.FLASH, TIME_OUT);
                if (!saveResult.isSuccess()) {
                    showLog("downloadTemplate failed \nCode : " + saveResult.code);
                } else {
                    showLog("downloadTemplate success\nPosition : " + position);
                }
            } finally {
                dismissProgressDialog();
            }
        }));

        findViewById(R.id.matchTemplate).setOnClickListener(v -> executor.execute(() -> {
            showFingerImage(null);
            showProgressDialog("Please press finger ");
            try {
                int position = getTemplatePosition();
                if (position < 0) {
                    return;
                }
                Result<Boolean> result = mxMscBigFingerApi.extractFingerFeatureBig(0, MxMscBigFingerApi.RAM, TIME_OUT);
                if (!result.isSuccess()) {
                    showLog("ExtractFeature \nCode  : " + result.code);
                    return;
                }
                Result<Boolean> booleanResult = mxMscBigFingerApi.match(0, MxMscBigFingerApi.RAM, position, MxMscBigFingerApi.FLASH);
                if (booleanResult.isSuccess()) {
                    showLog("MatchFinger success ! ");
                } else {
                    showLog("MatchFinger fail \nCode : " + booleanResult.code);
                }
            } finally {
                dismissProgressDialog();
            }
        }));

        findViewById(R.id.deleteTemplate).setOnClickListener(v -> executor.execute(() -> {
            showFingerImage(null);
            int templateIndex = getTemplatePosition();
            if (templateIndex < 0) {
                return;
            }
            Result<Boolean> deleteResult = mxMscBigFingerApi.deleteFlashTemplate(templateIndex);
            if (!deleteResult.isSuccess()) {
                showLog("DeleteTemplate failed \nCode : " + deleteResult.code);
            } else {
                showLog("DeleteTemplate success!");
            }
        }));

        findViewById(R.id.clearTemplate).setOnClickListener(v -> executor.execute(() -> {
            showFingerImage(null);
            Result<Boolean> clearResult = mxMscBigFingerApi.clearFlashTemplate();
            if (!clearResult.isSuccess()) {
                showLog("ClearTemplate failed \nCode : " + clearResult.code);
            } else {
                showLog("ClearTemplate success");
            }
        }));

        findViewById(R.id.searchTemplate).setOnClickListener(v -> {
            if (mSearchThread == null) {
                // start search
                mSearchThread = new SearchThread();
                ((TextView) v).setText(R.string.stop_search);
                setButtonsEnable(false);
                mSearchThread.start();
            } else {
                // stop search
                mSearchThread.interrupt();
                try {
                    mSearchThread.join();
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
                mSearchThread = null;
                ((TextView) v).setText(R.string.search);
                setButtonsEnable(true);
            }
        });
    }
    @Override
    protected void onDestroy() {
        super.onDestroy();
        UsbFingerprint.getInstance().UsbToHost();
    }
    class SearchThread extends Thread {

        @Override
        public void run() {
            super.run();
            int index = 0;
            while (!isInterrupted()) {
                // if you don't need show finger image , use this line .
                //Result<Integer> searchResult = mxMscBigFingerApi.findFingerTemplate();
                Result<Integer> searchResult = findTemplateWithImage();
                if (searchResult.isSuccess()) {
                    showLog("Count " + index++ + "\nSearch success !\nPosition :" + searchResult.data);
                } else {
                    String reason;
                    switch (searchResult.code) {
                        case MxMscBigFingerApi.CODE_NO_FINGER:
                            reason = "No Finger !";
                            break;
                        case MxMscBigFingerApi.CODE_FAIL:
                            reason = "Not Found !";
                            break;
                        default:
                            reason = "Error !";
                    }
                    showLog("Count " + index++ + "\nSearch failed  !\nReason  : " + reason);
                }
            }
        }
    }

    Result<Integer> findTemplateWithImage() {
        // detect finger
        Result<Boolean> detectFinger = mxMscBigFingerApi.detectFinger();
        if (!detectFinger.isSuccess()) {
            return Result.error(detectFinger.code);
        }
        if (detectFinger.data == null || !detectFinger.data) {
            return Result.error(MxMscBigFingerApi.CODE_NO_FINGER);
        }
        // read finger image to image cache
        Result<Boolean> readFingerImage = mxMscBigFingerApi.readFingerImage();
        if (!readFingerImage.isSuccess()) {
            return Result.error(readFingerImage.code);
        }
        // upload cache image to host
        Result<MxImage> fingerImageBig = mxMscBigFingerApi.uploadFingerImageBig();
        if (!fingerImageBig.isSuccess()) {
            return Result.error(fingerImageBig.code);
        }
        showFingerImage(fingerImageBig.data);
        // read finger feature to ram 0
        final int RAM_INDEX = 0;
        Result<Boolean> readFingerFeatureBig = mxMscBigFingerApi.readFingerFeatureBig(RAM_INDEX);
        if (!readFingerFeatureBig.isSuccess()) {
            return Result.error(readFingerFeatureBig.code);
        }
        return mxMscBigFingerApi.searchFlashTemplate(RAM_INDEX);
    }

    Result<Boolean> enrollTemplateWithHostFeature(int position) {
        //step 0 get finger image
        Result<MxImage> result = mxMscBigFingerApi.getFingerImageBig(TIME_OUT);
        if (!result.isSuccess()) {
            showFingerImage(null);
            return Result.error(result.code);
        }
        MxImage image = result.data;
        showFingerImage(image);
        //step 1 get finger feature
        assert image != null;
        byte[] featureResult = mxFingerAlg.extractFeature(image.data, image.width, image.height);
        if (featureResult == null) {
            return Result.error(999);
        }
        //step 2 down finger feature
        return mxMscBigFingerApi.downloadFlashTemplate(position, featureResult);
    }

    int getTemplatePosition() {
        EditText editText = findViewById(R.id.et_template);
        Editable text = editText.getText();
        try {
            int index = Integer.parseInt(text.toString());
            if (index > 127 || index < 0) {
                showLog("template position is beyond limits");
                return index;
            }
            return index;
        } catch (Exception e) {
            showLog("please input template position");
            return -1;
        }
    }

    void showProgressDialog(String msg) {
        runOnUiThread(() -> {
            showLog(msg);
            setButtonsEnable(false);
            findViewById(R.id.progress_horizontal).setVisibility(View.VISIBLE);
        });
    }

    void dismissProgressDialog() {
        runOnUiThread(() -> {
            setButtonsEnable(true);
            findViewById(R.id.progress_horizontal).setVisibility(View.GONE);
        });
    }

    void setButtonsEnable(boolean enable) {
        findViewById(R.id.getVersion).setEnabled(enable);
        findViewById(R.id.getSerialNumber).setEnabled(enable);
        findViewById(R.id.detectFinger).setEnabled(enable);
        findViewById(R.id.getFingerImage).setEnabled(enable);
        findViewById(R.id.enrollFingerHost).setEnabled(enable);
        findViewById(R.id.matchFingerHost).setEnabled(enable);
        findViewById(R.id.flashSize).setEnabled(enable);
        findViewById(R.id.matchTemplate).setEnabled(enable);
        findViewById(R.id.et_template).setEnabled(enable);
        findViewById(R.id.enrollTemplate).setEnabled(enable);
        findViewById(R.id.deleteTemplate).setEnabled(enable);
        findViewById(R.id.clearTemplate).setEnabled(enable);
        findViewById(R.id.searchTemplate).setEnabled(enable || mSearchThread != null);
    }

    void showFingerImage(MxImage image) {
        runOnUiThread(() -> {
            Bitmap bitmap = image == null ? null : RawBitmapUtils.raw2Bimap(image.data, image.width, image.height);
            ImageView imageView = findViewById(R.id.image);
            if (bitmap == null) {
                imageView.setVisibility(View.GONE);
            } else {
                imageView.setVisibility(View.VISIBLE);
                imageView.setImageBitmap(bitmap);
            }
        });
    }

    void appendLog(String msg) {
        Log.i(TAG, "appendLog: " + msg);
        runOnUiThread(() -> {
            TextView textView = findViewById(R.id.log);
            textView.append(msg);
            textView.append("\n");
        });
    }

    void showLog(String msg) {
        Log.i(TAG, "showLog: " + msg);
        runOnUiThread(() -> {
            TextView textView = findViewById(R.id.log);
            textView.setText(msg);
            textView.append("\n");
        });
    }
    private void checkReadWritePermission() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            if (checkSelfPermission(Manifest.permission.WRITE_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
                requestPermissions(new String[]{Manifest.permission.WRITE_EXTERNAL_STORAGE}, 0);
            }
            if (checkSelfPermission(Manifest.permission.READ_EXTERNAL_STORAGE) != PackageManager.PERMISSION_GRANTED) {
                requestPermissions(new String[]{Manifest.permission.READ_EXTERNAL_STORAGE}, 1);
            }
        }
    }


  Handler handler=new Handler(){
      @Override
      public void handleMessage(Message msg) {
          Toast.makeText(MainActivity.this, (String)msg.obj ,Toast.LENGTH_SHORT).show();
      }
  };

    private void writeFeatTemp(byte[] data){
        try {
            if(data==null){
                Message message= handler.obtainMessage();
                message.obj="保存指纹数据失败:data==null";
                handler.sendMessage(message);
                return;
            }
            File file = new File(featTempPath);
            FileWriter fileWriter = new FileWriter(file);
            fileWriter.write(StringUtility.bytesHexString(data,data.length));
            fileWriter.flush();
            fileWriter.close();

            Message message= handler.obtainMessage();
            message.obj="保存指纹数据成功:"+featTempPath;
            handler.sendMessage(message);

        }catch (Exception ex){

            Message message= handler.obtainMessage();
            message.obj="保存指纹数据失败:"+ex.toString();
            handler.sendMessage(message);

        }
    }

    private byte[] readFeatTemp(){
        File file=new File(featTempPath);
        if(!file.exists()){
            Message message= handler.obtainMessage();
            message.obj="没有指纹数据";
            handler.sendMessage(message);
            return null;
        }
        try {
            char[] data=new char[2048];
            FileReader fileReader=new FileReader(file);
            int len= fileReader.read(data);
            fileReader.close();
            if(len>0) {
                return StringUtility.hexString2Bytes(new String(data,0,len));
            }else{
                return null;
            }
        } catch (FileNotFoundException e) {
            Message message= handler.obtainMessage();
            message.obj="读取指纹数据异常:"+e.toString();
            handler.sendMessage(message);
        }catch (IOException e) {
            Message message= handler.obtainMessage();
            message.obj="读取指纹数据异常:"+e.toString();
            handler.sendMessage(message);
        }
        return null;
    }

}
